# -*- coding: utf-8 -*-
#
# Copyright (C) 2014-2015 Jaroslav Benkovský <jaroslav.benkovsky@nic.cz>
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution.

"""Trac plugin that provides CZ.NIC-specific operations for customizable
workflows.
"""

import os
from genshi.builder import tag
from pkg_resources import resource_filename
import re

from trac.core import implements, Component
from trac.web.api import IAuthenticator, IRequestFilter
from trac.web.chrome import ITemplateProvider, add_script
from acct_mgr.web_ui import LoginModule
from advancedworkflow.controller import TicketWorkflowOpBase


class FormatCheckLoginModule(LoginModule):
    """Allow logging only if the username matches "^[a-z]+$".
    This prevents logging with name.surname which was causing
    confusion with other trac users.
    To use this module, disable acct_mgr.web_ui.loginmodule."""

    implements(IAuthenticator)

    ## IAuthenticator methods
    re_uid = re.compile('^[a-z]+$')

    def authenticate(self, req):
        name = super(FormatCheckLoginModule, self).authenticate(req)
        if name and self.re_uid.match(name):
            return name
        else:
            return None


class UsualWorkflowJSInjector(Component):
    """Inject javascript to ticket.html pages that by default hides state transitions
    that are less common from the current ticket state."""

    implements(IRequestFilter, ITemplateProvider)

    ## IRequestFilter methods

    def pre_process_request(self, req, handler):
        return handler

    def post_process_request(self, req, template, data, content_type):
        if req.path_info.startswith('/newticket') or template != 'ticket.html':
            return (template, data, content_type)
        #if not data.get('ticket'):
        #    return (template, data, content_type)

        # JB: add a script that hides less common ticket state transitions, see #13312
        filename = 'cznicworkflow/js/usual_workflow.js'
        add_script(req, filename, 'text/javascript')

        return (template, data, content_type)

    ## ITemplateProvider methods

    def get_templates_dirs(self):
        return []

    def get_htdocs_dirs(self):
        return [('cznicworkflow', resource_filename('cznicworkflow', 'htdocs'))]




class TicketWorkflowOpFieldOwner(TicketWorkflowOpBase):
    """Sets the value of a ticket field to ticket owner, but
    only if owner is nonempty.

    <someaction>.operations = set_field_to_owner
    <someaction>.set_field_to_owner = myfield

    Don't forget to add the `TicketWorkflowOpFieldOwner` to the workflow
    option in [ticket].
    If there is no workflow option, the line will look like this:

    workflow = ConfigurableTicketWorkflow,TicketWorkflowOpFieldOwner
    """

    _op_name = 'set_field_to_owner'

    # ITicketActionController methods

    def render_ticket_action_control(self, req, ticket, action):
        """Returns the action control"""
        actions = self.get_configurable_workflow().actions
        label = actions[action]['name']
        field_name = self._get_field_name(action)
        if ticket['owner'] and ticket['owner'] != ticket[field_name]:
            hint = 'The %s will change to %s' % (field_name, ticket['owner'])
        else:
            hint = ''
        control = tag('')
        return label, control, hint

    def get_ticket_changes(self, req, ticket, action):
        """Returns the change of owner."""
        field_name = self._get_field_name(action)
        if ticket['owner'] and ticket['owner'] != ticket[field_name]:
            return {self._get_field_name(action): ticket['owner']}
        else:
            return {}

    def _get_field_name(self, action):
        """Determines the new owner"""
        # Should probably do some sanity checking...
        return self.config.get('ticket-workflow',
                                action + '.' + self._op_name).strip()


class TicketWorkflowOpOwnerFieldNotNull(TicketWorkflowOpBase):
    """Sets the value of the ticket owner to value of ticket field, but
    only if ticket field is nonempty.

    <someaction>.operations = set_owner_to_field
    <someaction>.set_owner_to_field = myfield

    Don't forget to add the `TicketWorkflowOpOwnerFieldNotNull` to the workflow
    option in [ticket].
    If there is no workflow option, the line will look like this:

    workflow = ConfigurableTicketWorkflow,TicketWorkflowOpOwnerFieldNotNull

    Note that the action's name is the same as in TicketWorkflowOpOwnerField,
    so only one of them might be loaded with workflow= option.
    """

    # Note that it's the same as in TicketWorkflowOpOwnerField
    _op_name = 'set_owner_to_field'

    # ITicketActionController methods

    def render_ticket_action_control(self, req, ticket, action):
        """Returns the action control"""
        actions = self.get_configurable_workflow().actions
        label = actions[action]['name']
        field_name = self._get_field_name(action)
        if ticket[field_name] and ticket['owner'] != ticket[field_name]:
            hint = 'The %s will change to %s' % ('owner', ticket[field_name])
        else:
            hint = ''
        control = tag('')
        return label, control, hint

    def get_ticket_changes(self, req, ticket, action):
        """Returns the change of owner."""
        field_name = self._get_field_name(action)
        if ticket[field_name] and ticket['owner'] != ticket[field_name]:
            return { 'owner': ticket[field_name] }
        else:
            return {}

    def _get_field_name(self, action):
        """Determines the new owner"""
        # Should probably do some sanity checking...
        return self.config.get('ticket-workflow',
                                action + '.' + self._op_name).strip()


class TicketWorkflowOpFieldSelf(TicketWorkflowOpBase):
    """Sets the value of a ticket field to current user, but
    only if current user is nonempty.

    <someaction>.operations = set_field_to_self
    <someaction>.set_field_to_self = myfield

    Don't forget to add the `TicketWorkflowOpFieldSelf` to the workflow
    option in [ticket].
    If there is no workflow option, the line will look like this:

    workflow = ConfigurableTicketWorkflow,TicketWorkflowOpFieldSelf
    """

    _op_name = 'set_field_to_self'

    # ITicketActionController methods

    def render_ticket_action_control(self, req, ticket, action):
        """Returns the action control"""
        actions = self.get_configurable_workflow().actions
        label = actions[action]['name']
        field_name = self._get_field_name(action)
        if req.authname and req.authname != ticket[field_name]:
            hint = 'The %s will change to %s' % (field_name, req.authname)
        else:
            hint = ''
        control = tag('')
        return label, control, hint

    def get_ticket_changes(self, req, ticket, action):
        """Returns the change of owner."""
        field_name = self._get_field_name(action)
        if req.authname and req.authname != ticket[field_name]:
            return {self._get_field_name(action): req.authname}
        else:
            return {}

    def _get_field_name(self, action):
        """Determines the new owner"""
        # Should probably do some sanity checking...
        return self.config.get('ticket-workflow',
                                action + '.' + self._op_name).strip()


